@@ -4,7 +4,6 @@
 package SVN::S4::Update;
 require 5.006_001;
 
-use SVN::S4;
 use strict;
 use Carp;
 use IO::Dir;
@@ -13,9 +12,11 @@ use Cwd;
 use Digest::MD5;
 use vars qw($AUTOLOAD);
 
+use SVN::S4;
+use SVN::S4::Debug qw (DEBUG is_debug);
 use SVN::S4::Path;
 
-our $VERSION = '1.050';
+our $VERSION = '1.052';
 our $Info = 1;
 
 
@@ -28,6 +29,7 @@ our $Info = 1;
 #######################################################################
 # OVERLOADS of S4 object
 package SVN::S4;
+use SVN::S4::Debug qw (DEBUG is_debug);
 
 sub update {
     my $self = shift;
@@ -35,30 +37,31 @@ sub update {
                   #paths=>,		#listref
 		  #fallback_cmd=>,	#listref
                   @_);
-    print "update: my params are: ", Dumper(\%params), "\n" if $self->debug;
+    DEBUG "update: my params are: ", Dumper(\%params), "\n" if $self->debug;
     my @paths = @{$params{paths}} if $params{paths};
     push @paths, "." if !@paths;
     if (!$params{fallback_cmd}) {  # used when called from within S4.
 	die "s4: %Error: s4 update needs revision" unless defined $params{revision};
-	my @cmd = ($self->{svn_binary}, 'update', @paths);
+	my @cmd = (split(/\s+/,$self->{svn_binary}), 'update', @paths);
 	push @cmd, ('--revision', $params{revision});
 	$params{fallback_cmd} = \@cmd;
     }
     if ($#paths > 0) {
 	#FIXME if any has a viewspec, barf.
         # punt on multiple params for now.
-        print "update with more than one arg. update normally\n" if $self->debug;
+        DEBUG "update with more than one arg. update normally\n" if $self->debug;
 	return $self->run ($params{fallback_cmd});
     }
+    my $abspath = $self->abs_filename($paths[0]);
 
     # see if a viewspec file is present
-    my $viewspec = "$paths[0]/" . $self->{viewspec_file};
-    my $found_viewspec = (-d "$paths[0]" && -f $viewspec);
+    my $viewspec = "$abspath/" . $self->{viewspec_file};
+    my $found_viewspec = $self->dir_uses_viewspec($abspath);
     if (!$found_viewspec) {
-        print "update tree with no viewspec. update normally\n" if $self->debug;
+        DEBUG "update tree with no viewspec ($viewspec). update normally\n" if $self->debug;
 	return $self->run ($params{fallback_cmd});
     }
-    print "Found a viewspec file. First update the top directory only.\n" if $self->debug;
+    DEBUG "Found a viewspec file. First update the top directory only.\n" if $self->debug;
 
     # Make final decision about what revision to update to.  This will be used
     # for all viewspec operations, so that you get a coherent rev number.
@@ -66,28 +69,32 @@ sub update {
     # switched, e.g. path1/path2/path3 where only path3 is switched, then
     # path1 and path2 may have revision numbers that are higher than $rev.)
     my $rev = $params{revision};
-    $rev = $self->which_rev (revision=>$rev, path=>$paths[0]);
-    print "Using revision $rev for all viewspec operations\n" if $self->debug;
+    $rev = $self->which_rev (revision=>$rev, path=>$abspath);
+    DEBUG "Using revision $rev for all viewspec operations\n" if $self->debug;
+    $self->{revision} = $rev;  # Force/override any user --revision flag
 
-    # Run update nonrecursively the first time.  The viewspec may replace
+    # We used to run update nonrecursively the first time.  The viewspec may replace
     # pieces of the tree, so it would sometimes be a big waste of time to
-    # update it all.
-    print "Updating the top\n" if $self->debug;
-    my @cmd_nonrecursive = ($self->{svn_binary}, "update", "--non-recursive", @paths);
+    # update it all.  However, this disallows a viewspec inside an existing checkout
+    # (or we'd have to repair the root, which confuses svn.)
+    DEBUG "Updating the top\n" if $self->debug;
+    my @cmd_nonrecursive = (split(/\s+/,$self->{svn_binary}), "update",
+			    #"--non-recursive",
+			    $abspath);
     push @cmd_nonrecursive, "--quiet" if $self->quiet;
     push @cmd_nonrecursive, ("-r$rev");
-    print "\t",join(' ',@cmd_nonrecursive),"\n" if $self->debug;
+    DEBUG "\t",join(' ',@cmd_nonrecursive),"\n" if $self->debug;
     local $! = undef;
     $self->run(@cmd_nonrecursive);
 
-    # did viewspec just disappear???
-    $found_viewspec = (-d "$paths[0]" && -f $viewspec);
+    # Did viewspec just disappear??? One hopes not.
+    $found_viewspec = $self->dir_uses_viewspec($abspath);
     if (!$found_viewspec) {
-        print "Viewspec disappeared. Do normal update.\n" if $self->debug;
-        print "viewspec was here, but now it's gone! update normally.\n" if $self->debug;
+        DEBUG "Viewspec disappeared. Do normal update.\n" if $self->debug;
+        DEBUG "viewspec was here, but now it's gone! update normally.\n" if $self->debug;
 	return $self->run ($params{fallback_cmd});
     }
-    print "Parse the viewspec file $viewspec\n" if $self->debug;
+    DEBUG "Parse the viewspec file $viewspec\n" if $self->debug;
     $self->parse_viewspec (filename=>$viewspec, revision=>$rev);
     # Test to see if a normal update will suffice, since it is a whole lot faster
     # than a bunch of individual updates.  For every successful checkout/update
@@ -95,12 +102,20 @@ sub update {
     # save it away.  Next time, if the viewspec hash is the same, and every
     # directory is scheduled to be updated to the same version (no rev NUM clauses),
     # then you can safely use a single update.
-    if (!$self->viewspec_changed(path=>$paths[0]) && $self->viewspec_compare_rev($rev)) {
-        print "viewspec is same as before. update normally.\n" if $self->debug;
-	return $self->run ($params{fallback_cmd});
+    if (!$self->viewspec_changed(path=>$abspath) && $self->viewspec_compare_rev($rev)) {
+        DEBUG "viewspec is same as before. update normally.\n" if $self->debug;
+	# We don't use fallback_cmd, as we want a specific revision
+	# Also, it breaks when given a symlink as a target, instead of the symlink's target
+	my $opt = SVN::S4::Getopt->new;
+	my @cmd = split(/\s+/,$self->{svn_binary});
+	push @cmd, $opt->formCmd('update', { %{$self},
+					     revision => $rev,
+					     path => [$abspath],
+					 });
+	return $self->run (@cmd);
     }
-    $self->apply_viewspec (path=>$paths[0]);
-    $self->save_viewspec_state (path=>$paths[0]);
+    $self->apply_viewspec (path=>$abspath);
+    $self->save_viewspec_state (path=>$abspath);
 }
 
 sub checkout {
@@ -113,18 +128,30 @@ sub checkout {
     if (!$params{fallback_cmd}) {  # used when called from within S4.
 	die "s4: %Error: s4 checkout needs url,path,revision"
 	   unless defined $params{url} && defined $params{path} && defined $params{revision};
-	my @cmd = ($self->{svn_binary}, 'checkout', $params{url}, $params{path});
+	my @cmd = (split(/\s+/,$self->{svn_binary}), 'checkout', $params{url}, $params{path});
 	push @cmd, ('--revision', $params{revision});
 	$params{fallback_cmd} = \@cmd;
     }
+
     # see if the area we're about the check out has a viewspec file
     my $viewspec_url = "$params{url}/$self->{viewspec_file}";
     my $found_viewspec = $self->is_file_in_repo (url => $viewspec_url);
+
+    # A checkout under an existing checkout is usually an error
+    # Some scripts expect this to be allowed, so we make it configurable
+    # However, viewspecs mess up, so never allow with a viewspec top
+    my $co_under_co = $self->config_get_bool('s4', 'co-under-co');
+    $co_under_co = 1 if !defined $co_under_co;
+    if (-e "$params{path}/.svn"
+	&& (!$co_under_co || $found_viewspec || -e "$params{path}/$self->{viewspec_file}")) {
+	    die "s4: %Error: Stubbornly refusing to checkout under existing checkout; you probably wanted 'update'\n";
+    }
+
     if (!$found_viewspec) {
-        print "checkout tree with no viewspec. checkout normally\n" if $self->debug;
+        DEBUG "checkout tree with no viewspec. checkout normally\n" if $self->debug;
 	return $self->run (@{$params{fallback_cmd}});
     }
-    print "Found a viewspec file in repo. First checkout the top directory only.\n" if $self->debug;
+    DEBUG "Found a viewspec file in repo. First checkout the top directory only.\n" if $self->debug;
 
     # Make final decision about what revision to update to.  This will be used
     # for all viewspec operations, so that you get a coherent rev number.
@@ -133,28 +160,31 @@ sub checkout {
     # path1 and path2 may have revision numbers that are higher than $rev.)
     my $rev = $params{revision};
     $rev = $self->which_rev (revision=>$rev, path=>$params{url});
-    print "Using revision $rev for all viewspec operations\n" if $self->debug;
+    DEBUG "Using revision $rev for all viewspec operations\n" if $self->debug;
 
-    # Run checkout nonrecursively the first time.  The viewspec may replace
+    # We used to run checkout nonrecursively the first time.  The viewspec may replace
     # pieces of the tree, so it would sometimes be a big waste of time to
-    # checkout it all.
-    print "s4: Checkout the top view directory into $params{path}\n" if $self->debug;
-    my @cmd_nonrecursive = ($self->{svn_binary}, "checkout", "--revision", $rev, "--non-recursive", $params{url}, $params{path});
+    # update it all.  However, this disallows a viewspec inside an existing checkout
+    # (or we'd have to repair the root, which confuses svn.)
+    DEBUG "s4: Checkout the top view directory into $params{path}\n" if $self->debug;
+    my @cmd_nonrecursive = (split(/\s+/,$self->{svn_binary}), "checkout", "--revision", $rev,
+			    #"--non-recursive",
+			    $params{url}, $params{path});
     push @cmd_nonrecursive, "--quiet" if $self->quiet;
-    print "\t",join(' ',@cmd_nonrecursive),"\n" if $self->debug;
+    DEBUG "\t",join(' ',@cmd_nonrecursive),"\n" if $self->debug;
     local $! = undef;
     $self->run(@cmd_nonrecursive);
     $self->wait_for_existence(path=>$params{path});
 
-    # Did viewspec just disappear??? One hiopes not.
+    # Did viewspec just disappear??? One hopes not.
     my $viewspec = "$params{path}/$self->{viewspec_file}";
-    $found_viewspec = (-d $params{path} && -f $viewspec);
+    $found_viewspec = $self->dir_uses_viewspec($params{path});
     if (!$found_viewspec) {
 	# I can only imagine this happening if checkout was interrupted, or somebody
 	# deleted the viewspec from the repo in the last few seconds.
 	die "s4: %Error: viewspec was in repo at $viewspec_url, but I could not find it in your checkout!";
     }
-    print "Parse the viewspec file $viewspec\n" if $self->debug;
+    DEBUG "Parse the viewspec file $viewspec\n" if $self->debug;
     $self->parse_viewspec (filename=>$viewspec, revision=>$rev);
     $self->apply_viewspec (path=>$params{path});
     $self->save_viewspec_state (path=>$params{path});
@@ -201,7 +231,7 @@ TBD
 
 The latest version is available from CPAN and from L<http://www.veripool.org/>.
 
-Copyright 2005-2010 by Bryce Denney.  This package is free software; you
+Copyright 2005-2011 by Bryce Denney.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 
